# -*- project-name: VASM -*-
package VASM::Resource::Action;

use strict;
use warnings;
use base qw/Exporter/;
use VASM::Action;
use VASM::Resource;
use VASM::Resource::Message;
use IO::Handle;
use URI;
use File::MimeInfo::Magic;

our @EXPORT = qw/findActionCatalog
                 mediaDirectory/;
our $VERSION = '1.04';

sub findActionCatalog {
  # A URI or absolute local filename. Note that 'file:///home/hanuman' is
  # different from '/home/hanuman', because many programs still refuse to
  # accept the URI form (such as qiv, for instance).
  my ($path) = @_; my ($scheme, $file, $mime);

  # If $path is a valid URI, good, try to find an actions catalog for it:
  if (defined ($scheme = URI->new($path)->scheme())) {
    $file = (findConfigResource('Action', 'URI', $scheme, 'Action.xml'))[0];
  } else { # By elimination, assume that this is a file
    return unless defined ($mime = mimetype($path));
    $file = (findConfigResource(
      'Action', 'File', (split m!/!, $mime), 'Action.xml'))[0];

    # If that fails, check each superclass of the mimetype
    unless (defined $file) {
      for my $superclass (File::MimeInfo::mimetype_isa($mime)) {
        $file = (findConfigResource(
          'Action', 'File', (split m!/!, $superclass), 'Action.xml'))[0];
        # Exit if satisfied
        last if defined $file;
      }
    }
  }

  return unless $file; # Return nothing unless a file was found, otherwise:
  return VASM::Action->new($file); # Digest file and return the new catalog
}

# Return a VASM::Message object describing all media types usable by
# VASM::Action, both for MIME and URI data:
sub mediaDirectory {
  my $catalog = findMessageCatalog('Action', 'Directory', $_[0]);

  # The benefits of encapsulation
  return $catalog;
}

# Snippet left here for possible future use:
#   fileno(makeConfigResource('vasm', @{ $args{Path} }, 'Action.xml')), 'w');

1;

__END__
